@@ -8,6 +8,7 @@ from typing import Any
 from vertx import EventBus
 from queue import Queue
 import traceback
+from redflagbpm.Services import Service, DocumentService, Context, ResourceService, ExecutionService, RuntimeService
 
 class ServiceError(Exception):
     def __init__(self, body:dict):
@@ -15,29 +16,45 @@ class ServiceError(Exception):
         self.trace = body.get("trace")
 
 class BPMService:
-    eb_calls: EventBus
-    eb_handlers: EventBus
+    service: Service
+    context: Context
+    execution: ExecutionService
+    documentService: DocumentService
+    resourceService: ResourceService
+    runtimeService: RuntimeService
+
+    __eb_calls: EventBus
+    __eb_handlers: EventBus
     delay = 10
     call_timeout = 30.0
-    handlers = []
-    address_dict: dict
+    __handlers = []
+    __address_dict: dict
 
     def __init__(self, host="localhost", port=7000, options=None, err_handler=None, ssl_context=None):
         if options is None:
             options = {}
-        self.eb_calls = EventBus(host=host, port=port, options=options, err_handler=err_handler,
-                                 ssl_context=ssl_context)
-        self.eb_handlers = EventBus(host=host, port=port, options=options, err_handler=err_handler,
-                                    ssl_context=ssl_context)
-        self.setupServices()
+        self.__eb_calls = EventBus(host=host, port=port, options=options, err_handler=err_handler,
+                                   ssl_context=ssl_context)
+        self.__eb_handlers = EventBus(host=host, port=port, options=options, err_handler=err_handler,
+                                      ssl_context=ssl_context)
+        self.service = Service(self)
+        self.context = Context(self)
+        self.execution = ExecutionService(self)
+        self.documentService = DocumentService(self)
+        self.resourceService = ResourceService(self)
+        self.runtimeService = RuntimeService(self)
+
         self.connect()
-        self.address_dict = {}
+        self.__address_dict = {}
         atexit.register(self.close)
 
-    def setupServices(self):
-        pass
-
-    def setAddress(self, address: Any = None):
+    def __setAddress(self, address: Any = None):
+        """
+        Este método se utiliza durante la conexión con el backend de la BPM.
+        Establece la dirección vertx habilitada en el backend para comunicarse con el contexto.
+        En el uso diario no es necesario invocarlo porque se autoconfigura.
+        :param address:
+        """
         if isinstance(address, dict):
             addr = address['replyAddress']
         elif isinstance(address, str):
@@ -47,15 +64,15 @@ class BPMService:
         else:
             raise ValueError('No address supplied')
         tid = threading.get_ident()
-        self.address_dict[tid] = addr
+        self.__address_dict[tid] = addr
 
-    def getAddress(self, address: Any = None) -> str:
+    def __getAddress(self, address: Any = None) -> str:
         if isinstance(address, dict):
             return address['replyAddress']
         elif isinstance(address, str):
             return address
-        elif threading.get_ident() in self.address_dict:
-            return self.address_dict[threading.get_ident()]
+        elif threading.get_ident() in self.__address_dict:
+            return self.__address_dict[threading.get_ident()]
         elif 'BPM_EVENT_BUS_REPLY' in os.environ:
             return os.environ['BPM_EVENT_BUS_REPLY']
         else:
@@ -63,21 +80,21 @@ class BPMService:
 
     def send(self, address, headers=None, body=None):
         ret = Queue()
-        self.eb_calls.send(address=address, headers=headers, body=body, reply_handler=lambda msg: ret.put(msg))
+        self.__eb_calls.send(address=address, headers=headers, body=body, reply_handler=lambda msg: ret.put(msg))
         return ret.get(True, self.call_timeout)
 
     def reply(self, body=None, address=None, headers=None):
-        addr = self.getAddress(address)
+        addr = self.__getAddress(address)
         if not isinstance(body, dict):
             body = {"reply": body}
-        self.eb_calls.send(address=addr, headers=headers, body=body)
+        self.__eb_calls.send(address=addr, headers=headers, body=body)
 
     def fail(self, message: str = None, address=None, headers=None):
         if message is None:
             message=traceback.format_exc()
-        addr = self.getAddress(address)
+        addr = self.__getAddress(address)
         body = {"error": message, "succeeded":False}
-        self.eb_calls.send(address=addr, headers=headers, body=body)
+        self.__eb_calls.send(address=addr, headers=headers, body=body)
 
     def call(self, address, body=None, headers=None):
         ret = self.send(address, headers, body)
@@ -90,49 +107,43 @@ class BPMService:
         return None
 
     def connect(self):
-        if not self.eb_calls.is_connected():
-            self.eb_calls.connect()
-        if not self.eb_handlers.is_connected():
-            self.eb_handlers.connect()
+        if not self.__eb_calls.is_connected():
+            self.__eb_calls.connect()
+        if not self.__eb_handlers.is_connected():
+            self.__eb_handlers.connect()
 
     def close(self):
-        for address in self.handlers:
+        for address in self.__handlers:
             self.unregister_handler(address)
-        if self.eb_calls.is_connected():
-            self.eb_calls.close()
-        if self.eb_handlers.is_connected():
-            self.eb_handlers.close()
+        if self.__eb_calls.is_connected():
+            self.__eb_calls.close()
+        if self.__eb_handlers.is_connected():
+            self.__eb_handlers.close()
 
-    def runtime_setVariable(self, processId: str, variables: dict):
-        ret = self.call("bpmHelper.runtime_setVariable", body={
-            "processId": processId,
-            "variables": variables})
-        return ret
-
-    def _run_external(self, address, handler):
+    def __run_external(self, address, handler):
         my_env = os.environ.copy()
         my_env["BPM_EVENT_BUS_REPLY"] = address["replyAddress"]
         subprocess.Popen(sys.executable+" "+handler, env=my_env, shell=True)
 
     def register_handler(self, address, handler):
         if isinstance(handler, str):
-            self.eb_handlers.register_handler(address, lambda msg: self._run_external(msg, handler))
+            self.__eb_handlers.register_handler(address, lambda msg: self.__run_external(msg, handler))
         else:
-            self.eb_handlers.register_handler(address, handler)
-        self.handlers.append(address)
+            self.__eb_handlers.register_handler(address, handler)
+        self.__handlers.append(address)
 
     def unregister_handler(self, address, handler=None):
-        self.eb_handlers.unregister_handler(address, handler)
-        self.handlers.remove(address)
+        self.__eb_handlers.unregister_handler(address, handler)
+        self.__handlers.remove(address)
 
     def run(self, address, handler):
         self.register_handler(address, handler)
         self.start()
 
     def exec(self, script: str, lang: str = 'javascript', context: dict = None, address: object = None) -> object:
-        addr = self.getAddress(address)
+        addr = self.__getAddress(address)
         ret = Queue()
-        self.eb_calls.send(address=addr, body={
+        self.__eb_calls.send(address=addr, body={
             "lang": lang,
             "script": script,
             "context": context
@@ -144,9 +155,9 @@ class BPMService:
             raise ValueError(result['body']['trace'])
 
     def request(self, request: str, address=None):
-        addr = self.getAddress(address)
+        addr = self.__getAddress(address)
         ret = Queue()
-        self.eb_calls.send(address=addr, body={"request": request}, reply_handler=lambda x: ret.put(x))
+        self.__eb_calls.send(address=addr, body={"request": request}, reply_handler=lambda x: ret.put(x))
         return ret.get(True, self.call_timeout)
 
     def start(self):
@@ -158,7 +169,7 @@ class BPMService:
             self.stop()
 
     def isConnected(self):
-        return self.eb_calls.is_connected()
+        return self.__eb_calls.is_connected()
 
     def stop(self):
         print("Stopping...")
