@@ -402,6 +402,6 @@ class ExecutionService:
         self.bpm.call("EBHBPMRuntime.removeVariable",
                       body={"executionId": process_instance_id, "variableName": variableName})
 
-    def getProcessInstanceId(self, variableName: str):
+    def getProcessInstanceId(self):
         process_instance_id = self.bpm.context.getValue('execution.getProcessInstanceId()')
         return process_instance_id
